@@ -3,7 +3,7 @@
 //
 // Contents: Routines that use connection handles
 //
-// Microsoft Drivers 5.9 for PHP for SQL Server
+// Microsoft Drivers 5.10 for PHP for SQL Server
 // Copyright(c) Microsoft Corporation
 // All rights reserved.
 // MIT License
@@ -30,6 +30,8 @@ extern "C" {
 
 namespace {
 
+const int MAX_CONN_VALSTRING_LEN = 256;
+
 // current subsytem.  defined for the CHECK_SQL_{ERROR|WARNING} macros
 unsigned int current_log_subsystem = LOG_CONN;
 
@@ -38,13 +40,7 @@ struct date_as_string_func {
     static void func( connection_option const* /*option*/, _In_ zval* value, _Inout_ sqlsrv_conn* conn, std::string& /*conn_str*/ )
     {
         ss_sqlsrv_conn* ss_conn = static_cast<ss_sqlsrv_conn*>( conn );
-
-        if( zend_is_true( value )) {
-            ss_conn->date_as_string = true;
-        }
-        else {
-            ss_conn->date_as_string = false;
-        }
+        ss_conn->date_as_string = zend_is_true(value);
     }
 };
 
@@ -53,19 +49,12 @@ struct format_decimals_func
     static void func(connection_option const* /*option*/, _In_ zval* value, _Inout_ sqlsrv_conn* conn, std::string& /*conn_str*/)
     {
         ss_sqlsrv_conn* ss_conn = static_cast<ss_sqlsrv_conn*>(conn);
-
-        if (zend_is_true(value)) {
-            ss_conn->format_decimals = true;
-        }
-        else {
-            ss_conn->format_decimals = false;
-        }
+        ss_conn->format_decimals = zend_is_true(value);
     }
 };
 
 struct decimal_places_func
 {
-
     static void func(connection_option const* /*option*/, _In_ zval* value, _Inout_ sqlsrv_conn* conn, std::string& /*conn_str*/)
     {
         // first check if the input is an integer
@@ -83,6 +72,33 @@ struct decimal_places_func
     }
 };
 
+struct srv_encrypt_set_func {
+    static void func(connection_option const* option, _In_ zval* value_z, _Inout_ sqlsrv_conn* conn, std::string& conn_str)
+    {
+        std::string attr;
+
+        if (Z_TYPE_P(value_z) == IS_LONG) {
+            long val = Z_LVAL_P(value_z);
+            if (val == 1) {
+                attr = "yes";
+            } else if (val == 0) {
+                attr = "no";
+            } else {
+                attr = std::to_string(val);
+            }
+        } else if (Z_TYPE_P(value_z) == IS_TRUE || Z_TYPE_P(value_z) == IS_FALSE) {
+            attr = zend_is_true(value_z) ? "yes" : "no";
+        } else {
+            attr = Z_STRVAL_P(value_z);
+        }
+
+        char temp_str[MAX_CONN_VALSTRING_LEN];
+        snprintf(temp_str, MAX_CONN_VALSTRING_LEN, "%s={%s};", option->odbc_name, attr.c_str());
+
+        conn_str += temp_str;
+    }
+};
+
 struct conn_char_set_func {
 
     static void func( connection_option const* /*option*/, _Inout_ zval* value, _Inout_ sqlsrv_conn* conn, std::string& /*conn_str*/ )
@@ -117,17 +133,10 @@ struct bool_conn_str_func {
 
     static void func( _In_ connection_option const* option, _In_ zval* value, sqlsrv_conn* /*conn*/, _Out_ std::string& conn_str )
     {
-        char const* val_str;
-        if( zend_is_true( value )) {
-            val_str = "yes";
-        }
-        else {
-            val_str = "no";
-        }
-        conn_str += option->odbc_name;
-        conn_str += "={";
-        conn_str += val_str;
-        conn_str += "};";
+        char temp_str[MAX_CONN_VALSTRING_LEN];
+
+        snprintf(temp_str, MAX_CONN_VALSTRING_LEN, "%s={%s};", option->odbc_name, (zend_is_true(value) ? "yes" : "no"));
+        conn_str += temp_str;
     }
 };
 
@@ -137,12 +146,10 @@ struct int_conn_str_func {
     {
         SQLSRV_ASSERT( Z_TYPE_P( value ) == IS_LONG, "An integer is expected for this keyword" ) 
 
-        std::string val_str = std::to_string( Z_LVAL_P( value ));
-        
-        conn_str += option->odbc_name;
-        conn_str += "={";
-        conn_str += val_str;
-        conn_str += "};";
+        char temp_str[MAX_CONN_VALSTRING_LEN];
+
+        snprintf(temp_str, MAX_CONN_VALSTRING_LEN, "%s={%ld};", option->odbc_name, Z_LVAL_P(value));
+        conn_str += temp_str;
     }
 };
 
@@ -251,6 +258,8 @@ const char TransactionIsolation[] = "TransactionIsolation";
 const char TransparentNetworkIPResolution[] = "TransparentNetworkIPResolution";
 const char UID[] = "UID";
 const char WSID[] = "WSID";
+const char ComputePool[] = "ComputePool";
+const char HostNameInCertificate[] = "HostNameInCertificate";
 
 }
 
@@ -440,8 +449,8 @@ const connection_option SS_CONN_OPTS[] = {
         SQLSRV_CONN_OPTION_ENCRYPT,
         ODBCConnOptions::Encrypt, 
         sizeof( ODBCConnOptions::Encrypt ),
-        CONN_ATTR_BOOL,
-        bool_conn_str_func::func
+        CONN_ATTR_MIXED,
+        srv_encrypt_set_func::func
     },
     { 
         SSConnOptionNames::Failover_Partner,
@@ -596,6 +605,25 @@ const connection_option SS_CONN_OPTS[] = {
         CONN_ATTR_INT,
         decimal_places_func::func
     },
+    {
+        SSConnOptionNames::ComputePool,
+        sizeof(SSConnOptionNames::ComputePool),
+        SQLSRV_CONN_OPTION_COMPUTE_POOL,
+        ODBCConnOptions::ComputePool,
+        sizeof(ODBCConnOptions::ComputePool),
+        CONN_ATTR_STRING,
+        conn_str_append_func::func
+    },
+    {
+        SSConnOptionNames::HostNameInCertificate,
+        sizeof(SSConnOptionNames::HostNameInCertificate),
+        SQLSRV_CONN_OPTION_HOSTNAME_IN_CERT,
+        ODBCConnOptions::HostNameInCertificate,
+        sizeof(ODBCConnOptions::HostNameInCertificate),
+        CONN_ATTR_STRING,
+        conn_str_append_func::func
+    },
+
     { NULL, 0, SQLSRV_CONN_OPTION_INVALID, NULL, 0 , CONN_ATTR_INVALID, NULL },  //terminate the table
 };
 
@@ -1355,6 +1383,8 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
                     // if we ever introduce a boolean connection option that maps to a string connection
                     // attribute.
                     break;
+                case CONN_ATTR_MIXED:
+                    break;
                 case CONN_ATTR_INT:
                 {
                     CHECK_CUSTOM_ERROR( (Z_TYPE_P( value_z ) != IS_LONG ), ctx, SQLSRV_ERROR_INVALID_OPTION_TYPE_INT,
@@ -1381,16 +1411,6 @@ int get_conn_option_key( _Inout_ sqlsrv_context& ctx, _In_ zend_string* key, _In
                         throw ss::SSException();
                     }
 
-                    bool valid = true;
-                    if( stricmp( SS_CONN_OPTS[i].sqlsrv_name, SSConnOptionNames::Authentication ) == 0 ) {
-                        valid = AzureADOptions::isAuthValid(value, value_len);
-                    }
-
-                    CHECK_CUSTOM_ERROR( !valid, ctx, SS_SQLSRV_ERROR_INVALID_AUTHENTICATION_OPTION, SS_CONN_OPTS[i].sqlsrv_name ) {
-
-                        throw ss::SSException();
-                    }
-
                     break;
                 }
                 case CONN_ATTR_INVALID:
